package java.beans;

import java.awt.Image;
import java.lang.ref.SoftReference;
import java.lang.ref.Reference;

class GenericBeanInfo extends SimpleBeanInfo
{
    private BeanDescriptor beanDescriptor;
    private EventSetDescriptor[] events;
    private int defaultEvent;
    private PropertyDescriptor[] properties;
    private int defaultProperty;
    private MethodDescriptor[] methods;
    private Reference<BeanInfo> targetBeanInfoRef;
    
    public GenericBeanInfo(BeanDescriptor beanDescriptor, EventSetDescriptor[] events, int defaultEvent, PropertyDescriptor[] properties, int defaultProperty, MethodDescriptor[] methods, BeanInfo referent) {
        this.beanDescriptor = beanDescriptor;
        this.events = events;
        this.defaultEvent = defaultEvent;
        this.properties = properties;
        this.defaultProperty = defaultProperty;
        this.methods = methods;
        this.targetBeanInfoRef = ((referent != null) ? new SoftReference(referent) : null);
    }
    
    GenericBeanInfo(GenericBeanInfo genericBeanInfo) {
        this.beanDescriptor = new BeanDescriptor(genericBeanInfo.beanDescriptor);
        if (genericBeanInfo.events != null) {
            int length = genericBeanInfo.events.length;
            this.events = new EventSetDescriptor[length];
            for (int i = 0; i < length; ++i) {
                this.events[i] = new EventSetDescriptor(genericBeanInfo.events[i]);
            }
        }
        this.defaultEvent = genericBeanInfo.defaultEvent;
        if (genericBeanInfo.properties != null) {
            int length2 = genericBeanInfo.properties.length;
            this.properties = new PropertyDescriptor[length2];
            for (int j = 0; j < length2; ++j) {
                PropertyDescriptor propertyDescriptor = genericBeanInfo.properties[j];
                if ((propertyDescriptor instanceof IndexedPropertyDescriptor)) {
                    this.properties[j] = new IndexedPropertyDescriptor((IndexedPropertyDescriptor)propertyDescriptor);
                }
                else {
                    this.properties[j] = new PropertyDescriptor(propertyDescriptor);
                }
            }
        }
        this.defaultProperty = genericBeanInfo.defaultProperty;
        if (genericBeanInfo.methods != null) {
            int length3 = genericBeanInfo.methods.length;
            this.methods = new MethodDescriptor[length3];
            for (int k = 0; k < length3; ++k) {
                this.methods[k] = new MethodDescriptor(genericBeanInfo.methods[k]);
            }
        }
        this.targetBeanInfoRef = genericBeanInfo.targetBeanInfoRef;
    }
    
    @Override
    public PropertyDescriptor[] getPropertyDescriptors() {
        return this.properties;
    }
    
    @Override
    public int getDefaultPropertyIndex() {
        return this.defaultProperty;
    }
    
    @Override
    public EventSetDescriptor[] getEventSetDescriptors() {
        return this.events;
    }
    
    @Override
    public int getDefaultEventIndex() {
        return this.defaultEvent;
    }
    
    @Override
    public MethodDescriptor[] getMethodDescriptors() {
        return this.methods;
    }
    
    @Override
    public BeanDescriptor getBeanDescriptor() {
        return this.beanDescriptor;
    }
    
    @Override
    public Image getIcon(int n) {
        BeanInfo targetBeanInfo = this.getTargetBeanInfo();
        if (targetBeanInfo != null) {
            return targetBeanInfo.getIcon(n);
        }
        return super.getIcon(n);
    }
    
    private BeanInfo getTargetBeanInfo() {
        if (this.targetBeanInfoRef == null) {
            return null;
        }
        BeanInfo referent = (BeanInfo)this.targetBeanInfoRef.get();
        if (referent == null) {
            referent = (BeanInfo)ThreadGroupContext.getContext().getBeanInfoFinder().find(this.beanDescriptor.getBeanClass());
            if (referent != null) {
                this.targetBeanInfoRef = new SoftReference(referent);
            }
        }
        return referent;
    }
}
